@@ -76,8 +76,8 @@ extern tcl_timer_t *timer, *utimer;
    modified versions of this bot.
 
  */
-char egg_version[1024] = "1.3.0";
-int egg_numver = 1030000;
+char egg_version[1024] = "1.3.1";
+int egg_numver = 1030100;
 
 /* person to send a note to for new users */
 char notify_new[121] = "";
@@ -264,6 +264,8 @@ static void got_bus (int z)
    fatal("BUS ERROR -- CRASHING!", 1);
 #ifdef EBUG
    kill(getpid(),SIGBUS);
+#else
+   exit(1);
 #endif
 }
 
@@ -273,6 +275,8 @@ static void got_segv (int z)
    fatal("SEGMENT VIOLATION -- CRASHING!", 1);
 #ifdef EBUG
    kill(getpid(),SIGSEGV);
+#else
+   exit(1);
 #endif
 }
 
@@ -546,11 +550,17 @@ int main (int argc, char ** argv)
    /* set up error traps: */
    sv.sa_handler = got_bus;
    sigemptyset(&sv.sa_mask);
+#ifdef SA_ONESHOT
    sv.sa_flags = SA_ONESHOT;
+#else
+   sv.sa_flags = 0;
+#endif
    sigaction(SIGBUS, &sv, NULL);
    sv.sa_handler = got_segv;
    sigaction(SIGSEGV, &sv, NULL);
+#ifdef SA_ONESHOT
    sv.sa_flags = 0;
+#endif
    sv.sa_handler = got_fpe;
    sigaction(SIGFPE, &sv, NULL);
    sv.sa_handler = got_term;
